
const express = require("express");
const createError = require("http-errors");
const router = express.Router();
const addPage = require("../models/new-page");

//get post
router.get('/pages', async (req, res, next) => {
  try {
    const data = await addPage.find().sort({ created_at: -1 })
    const total_post = await addPage.countDocuments();
    res.send({
      total: total_post,
      addPage: data,
    });
  }
  catch (error) {
    next(error)
  }
})
//Get data by slug
router.get("/page_by_slug/:slug", async (req, res, next) => {
  try {
    const data = await addPage.find({ slug: req.params.slug })
    res.send(data)
  }
  catch (error) {
    next(error)
  }
});

// Get data by slug for next and preview
router.get("/page_by_slug_n_p/:slug", async (req, res, next) => {
  try {
    const data = await addPage.find({ slug: req.params.slug })
    let data_next = 'erorr';
    let data_prev = 'erorr';
    if (data) {
      const data_prev = await addPage.find({ _id: { $gt: data[0]._id } }).sort({ _id: 1 }).limit(1);
      if (data_prev) {
        const data_next = await addPage.find({ _id: { $lt: data[0]._id } }).sort({ _id: -1 }).limit(1);
        res.send({ result: { data_prev, data_next } });
      } else {
        res.send({ result: { data_prev, data_next } });
      }
    } else {
      res.send({ result: { data_prev, data_next } });
    }

  }
  catch (error) {
    next(error)
  }
});

//get releted blog
router.get('/pages/slug/:slug', (req, res) => {
  const slug = req.params.slug;
  // Fetch related pages from MongoDB based on the category
  addPage.find({ slug: slug })
    .sort({ createdAt: -1 })
    .exec((err, slugpost) => {
      if (err) {
        console.error('Error retrieving related pages from MongoDB', err);
        res.status(500).send('Internal Server Error');
      } else {
        res.json(slugpost);
      }
    });
});

router.get('/recentposts', (req, res) => {
  // Fetch the five most recent pages from MongoDB
  addPage.find()
    .sort({ created_at: -1 })
    .limit(5)
    .exec((err, recentposts) => {
      if (err) {
        console.error('Error retrieving pages from MongoDB', err);
        res.status(500).send('Internal Server Error');
      } else {
        res.json(recentposts);
      }
    });
});


// Items route
router.get('/api/items', async (req, res) => {
  try {
    const page = parseInt(req.query.page) || 1;
    const limit = parseInt(req.query.limit) || 5;
    const totalItems = await addPage.countDocuments();
    const totalPages = Math.ceil(totalItems / limit);
    const currentPage = Math.min(Math.max(page, 1), totalPages);

    const items = await addPage
      .find({}).sort({ created_at: -1 })
      .limit(limit)
      .skip((currentPage - 1) * limit)
    res.status(200).json({
      items,
      totalItems,
      totalPages,
      currentPage,
      itemsPerPage: limit
    });
  } catch (error) {
    console.error(error);
    res.status(500).send('Error retrieving items');
  }
});


router.get("/getpostsearch", async (req, res, next) => {
  try {
    const limit = parseInt(req.query.limit);
    const offset = parseInt(req.query.offset);
    const search = String(req.query.searchdata);
    const skips = limit * offset;

    var sortObject = {};
    var sort_col = String(req.query.sort_col);
    var sort_order = String(req.query.sort_order);
    sortObject[sort_col] = sort_order;
    if (search) {
      const colName = search;
      const postCollection = await addPage.find({
        $or: [
          { name: { $regex: new RegExp("^" + colName.toLowerCase(), "i") } },
        ],
      })
        .skip(skips)
        .limit(limit)
        .sort(sortObject);
      const total_pages = await addPage.countDocuments({
        $or: [
          { name: { $regex: new RegExp("^" + colName.toLowerCase(), "i") } }
        ],
      });
      res.send({
        total: total_pages,
        data: postCollection,
      });
    } else {
      const postCollection = await addPage.find()
        .skip(skips)
        .limit(limit)
        .sort(sortObject);
      const total_pages = await addPage.countDocuments();
      res.send({
        total: total_pages,
        data: postCollection,
      });
    }
  } catch (error) {
    next(error);
  }
});
//get addPage by ID
router.get('/pages/:id', async (req, res, next) => {

  try {
    const data = await addPage.find({ _id: req.params.id })
    res.send({ data: data })

  }
  catch (error) {
    next(error)
  }
})
//Create post
router.post("/pages", async (req, res, next) => {
  try {
    const {
      page_title,
      page_content,
      metatitle,
      metadescription,
      focuskeyword,
      faqs,
      selectedImagid,
      imagefilePath,
      img_alt,
      slug,
      status,
      created_at,
      updated_at,
    } = req.body;
    const mypost = new addPage({
      page_title,
      page_content,
      metatitle,
      metadescription,
      focuskeyword,
      faqs,
      selectedImagid,
      imagefilePath,
      img_alt,
      slug,
      status,
      created_at,
      updated_at,
    });
    const addpage = await mypost.save();
    res.send(addpage);
  } catch (error) {
    next(error);
  }
});
//Update post
router.patch("/pages/:id", async (req, res, next) => {
  try {
    const update = req.body;
    const id = req.params.id;
    const option = { new: true };
    const updatedDetails = await addPage.findByIdAndUpdate(id, update, option);
    res.send(updatedDetails);
  } catch (error) {
    next(error);
  }
});
//Delete post
router.delete("/pages/:id", async (req, res, next) => {
  try {
    const id = req.params.id;
    const find = await addPage.findOne({ _id: id });
    if (find) {
      const deleted = await addPage.findByIdAndDelete({ _id: id });
      res.send(deleted);
    } else {
      res.send("Id does not exist");
    }
  } catch (error) {
    next(error)
  }
});
router.get('/checkurl/:slug', async (req, res, next) => {
  try {
    const data = await addPage.find({ slug: req.params.slug })
    res.send({ data: data })
  }
  catch (error) {
    next(error)
  }
})

//related post by category
router.get('/pages/category/:category', (req, res) => {
  const category = req.params.category;
  addPage.find({ post_category: category })
    .sort({ createdAt: -1 }).limit(3)
    .exec((err, reletedposts) => {
      if (err) {
        console.error('Error retrieving related pages from MongoDB', err);
        res.status(500).send('Internal Server Error');
      } else {
        res.json(reletedposts);
      }
    });
});

// Get post by id for filter
router.get('/category_Slugh', async (req, res) => {
  try {
    const dataCategory = await addPage.find({ category_id: req.query.id })
    res.send({ dataCategory })
  }
  catch (error) {

  }
});
// Get post by id for filter
router.get('/blogs_by_slug/:slug', async (req, res) => {
  try {
    var slug = req.params.slug;
    // Finding courses of category Database
    postCategory.find({ slug: slug })
      .then(data => {
        addPage.find({ category_id: { $regex: data[0]._id, $options: 'i' } })
          .then(res_data => {
            // res.send(res_data);
            res.send({
              data: res_data,
            });
          })
          .catch(error => {
            console.log(error);
          })
      })
      .catch(error => {
        console.log(error);
      });
  }
  catch (error) {

  }
});

//===================== Get tag data by slug ================================

router.get('/blogs_by_slug_tag/:slug', async (req, res) => {
  try {
    var slug = req.params.slug;
    postTags.find({ slug: slug })
      .then(data => {
        addPage.find({ tagdata_id: { $regex: data[0]._id, $options: 'i' } })
          .then(res_data_tag => {
            res.send({
              data: res_data_tag,
            });
          })
          .catch(error => {
            console.log(error);
          })
      })
      .catch(error => {
        console.log(error);
      });
  }
  catch (error) {

  }
});

//===================== Get author data by slug ================================

router.get('/blogs_by_slug_author/:slug', async (req, res) => {
  try {
    var slug = req.params.slug;
    Account.find({ slug: slug })
      .then(data => {
        addPage.find({ authorProfile_id: { $regex: data[0]._id, $options: 'i' } })
          .then(res_data_author => {
            res.send({
              data: res_data_author,
            });
          })
          .catch(error => {
            console.log(error);
          })
      })
      .catch(error => {
        console.log(error);
      });
  }
  catch (error) {

  }
});

module.exports = router;
