const express = require('express');
const mongoose = require('mongoose');
var bodyParser = require('body-parser');
const dotenv = require('dotenv');
var cors = require('cors');
const path = require('path');
const fs = require('fs');
const multer = require('multer');


//require('dotenv').config();
require("../db/connection");
const app = new express();
const AddPostRoute = require("./routes/add-post.route");
const PostcategoryRoute = require("./routes/post-category.route");
const PostTagRoute = require("./routes/post-tags.route");
const FooterOneRoute = require("./routes/footer-wedget-one.route");
const FooterThreeRoute = require("./routes/footer-wedget-three.route");
const CommentRoute = require("./routes/comments.route");
const NewPageRoute = require("./routes/new-page.route");
const HeaderMenuRoute = require("./routes/header-menu.route");
const AccountRoute = require("./routes/account.route");
const LogotRoute = require("./routes/mylogo.route");
const AdvertisementRoute = require("./routes/advertisement.route");

// app.use(express.static(path.join(__dirname + "/uploads")))
app.use(bodyParser.urlencoded({ extended: false }))
app.use(cors());
// parse application/json
app.use(bodyParser.json())
app.use(express.json());
app.use(
  AddPostRoute,
  PostcategoryRoute,
  PostTagRoute,
  FooterOneRoute,
  FooterThreeRoute,
  CommentRoute,
  NewPageRoute,
  HeaderMenuRoute,
  AccountRoute,
  LogotRoute,
  AdvertisementRoute
);
app.use(function (req, res, next) {
  res.header("Access-Control-Allow-Origin", "*");
  res.header("Access-Control-Allow-Headers", "Origin, X-Requested-With, Content-Type, Accept");
  next();
});
var storage = multer.diskStorage({
  // destination
  destination: function (req, file, cb) {
    cb(null, '../src/assets/images')
  },
  filename: function (req, file, cb) {
     const fileName = `${Date.now()}-${file.originalname}`;
    cb(null, fileName);
  }
});
const upload = multer({ storage });

//======================= image upload here===========================
const ImageSchema = new mongoose.Schema({
  filename: String,
  path: String,
  created_at: {
    type: Date,
    default: Date.now
  }
});

const Image = mongoose.model('Image', ImageSchema);
app.get('/uploadsImage', async (req, res, next) => {
  try {
    const data = await Image.find().sort({ created_at: -1 })
    const total_images = await Image.countDocuments();
    res.send({
      images: data,
      total:total_images
    });
  }
  catch (error) {
    next(error)
  }
})

// get data for pegination
app.get('/images/items', async (req, res) => {
  try {
      const page = parseInt(req.query.page) || 1;
      const limit = parseInt(req.query.limit) || 5;
      const totalItems = await Image.countDocuments();
      const totalPages = Math.ceil(totalItems / limit);
      const currentPage = Math.min(Math.max(page, 1), totalPages);

      const items = await Image
          .find({}).sort({ created_at: -1 })
          .limit(limit)
          .skip((currentPage - 1) * limit)
      res.status(200).json({
          items,
          totalItems,
          totalPages,
          currentPage,
          itemsPerPage: limit
      });
  } catch (error) {
      console.error(error);
      res.status(500).send('Error retrieving items');
  }
});

// app.get("/getimagesearch", async (req, res, next) => {
//   try {
//     const limit = parseInt(req.query.limit);
//     const offset = parseInt(req.query.offset);
//     const search = String(req.query.searchdata);
//     const skips = limit * offset;

//     var sortObject = {};
//     var sort_col = String(req.query.sort_col);
//     var sort_order = String(req.query.sort_order);
//     sortObject[sort_col] = sort_order;
//     if (search) {
//       const colName = search;
//       const imgCollection = await Image.find({
//         $or: [
//           { filename: { $regex: new RegExp("^" + colName.toLowerCase(), "i") } },
//         ],
//       })
//         .skip(skips)
//         .limit(limit)
//         .sort(sortObject);
//       const total_pages = await Image.countDocuments({
//         $or: [
//           { filename: { $regex: new RegExp("^" + colName.toLowerCase(), "i") } }
//         ],
//       });
//       res.send({
//         total: total_pages,
//         data: imgCollection,
//       });
//     } else {
//       const imgCollection = await Image.find()
//         .skip(skips)
//         .limit(limit)
//         .sort(sortObject);
//       const total_pages = await Image.countDocuments();
//       res.send({
//         total: total_pages,
//         data: imgCollection,
//       });
//     }
//   } catch (error) {
//     next(error);
//   }
// });

const escapeRegex = (text) => {
  return text.replace(/[-[\]{}()*+?.,\\^$|#\s]/g, '\\$&');
};

app.get("/getimagesearch", async (req, res, next) => {
  try {
    const limit = parseInt(req.query.limit);
    const offset = parseInt(req.query.offset);
    const search = req.query.searchdata ? escapeRegex(String(req.query.searchdata)) : null;
    const skips = limit * offset;

    const sortObject = {};
    const sort_col = String(req.query.sort_col);
    const sort_order = String(req.query.sort_order);
    sortObject[sort_col] = sort_order;

    let query = {};

    if (search) {
      query = {
        filename: { $regex: new RegExp(escapeRegex(search), "i") }
      };
    }

    const imgCollection = await Image.find(query)
      .skip(skips)
      .limit(limit)
      .sort(sortObject);

    const total_pages = await Image.countDocuments(query);

    res.send({
      total: total_pages,
      data: imgCollection,
    });
  } catch (error) {
    next(error);
  }
});


//Delete featured images
app.delete("/delete_Featured_images/:id", async (req, res, next) => {
  try {
    const id = req.params.id;
    const find = await Image.findOne({ _id: id });
    if (find) {
      const deleted = await Image.findByIdAndDelete({ _id: id });
      res.send(deleted);
    } else {
      res.send("Id does not exist");
    }
  } catch (error) {
    next(error)
  }
});

//get Image data by id 
app.get('/getimagesbyid', async (req, res, next) => {
  try {
    const id= req.query.image_id;
    const data = await Image.find({_id:id})
    res.send({
      images: data,
    });
  }
  catch (error) {
    next(error)
  }
})

app.use(express.static('public'));
app.use(express.json());

app.post('/api/upload_gallery', upload.single('image'), async (req, res) => {
  try {
    const { filename} = req.file;
    const modifiedPath = `/assets/images/${filename}`;
    const image = new Image({ filename, path: modifiedPath });
    await image.save();
    res.json({ message: 'File uploaded and saved to database.' });
  } catch (error) {
    console.error(error);
    res.status(500).json({ message: 'Error uploading file.' });
  }
});


// var upload = multer({ storage: storage });
// app.post('/api/upload_gallery', upload.array("myFile[]", 12), function (req, res) {
//   res.send('Image saved');

// });

app.listen(3000, () => {
  console.log(`Server Started at ${3000}`)
})
